#!/bin/sh

# mac-detect.sh - Part of the ESP package. A utility to report on active MAC addresses
#                 that are not registered in the ESP dhcpd.conf file. These 
#                 may be the result of a user bypassing DHCP and assigning a 
#                 hard-coded IP.
#
# This utility should run on the ESP server as root. It works as follows:
# 1) NMAP scan of entire subnet to generate ARP request traffic on every possible 
#    host.
# 2) Arpwatch records MAC-IP info generated above.
# 3) Arpwatch database is compared to dhcpd.conf - MACs found in the former but not
#    the latter are recorded.
# 4) Result is compared to 'exclude' file. MAC info not in exclude file is printed to
#    output text file.
#
# Additional file arp_watch.debug is generated which contains more details of
# all MACs found. mac-detect.sh takes about 30 sec to run on a 256 host subnet. 
#
# User Configuration Area
#

# Fully qualified path to dhcpd.conf.
DHCPDCONF=/etc/dhcpd/dhcpd.conf

# Fully qualified path to output text file.
OUT_FILE=/var/log/mac-rpt.txt

# Fully qualified path to arpwatch.
ARPWATCH=/usr/sbin/arpwatch
 
# Fully qualified path to nmap.
NMAP=/usr/bin/nmap

# Network to check.
NET="192.168.1.0/24"
#NET=

# Name of exclude file.
EXCLUDE=/root/exclude

# End of user configuration.
 
export DHCPDCONF
export OUT_FILE
export EXCLUDE
DIR=`pwd`
MAP="$DIR/arp_watch.map"
DEBUG_OUTPUT="$DIR/arp_watch.debug"
TEMPFILE1="$DIR/tempone$$"
export TEMPFILE1
TEMPFILE2="$DIR/temptwo$$"
export TEMPFILE2

if [ -z $NET ] 
   then
   echo "Network not entered - exiting."
   exit
fi

killall -q  $ARPWATCH
sleep 5

# zero-ing resultant files
echo -n "" >  $MAP
echo -n "" > $DEBUG_OUTPUT

$ARPWATCH -dN -f $MAP -n $NET > $DEBUG_OUTPUT &
$NMAP  --packet_trace  -PT135 --max_rtt_timeout 20 $NET >/dev/null 2>&1

sleep 3
killall -HUP -q  $ARPWATCH

# pad the arpwatch MAC output with leading zeros
 
perl -e '
{
   while(<>){

      if (/(\S*)\:(\S*)\:(\S*)\:(\S*)\:(\S*)\:(\S*)(.*)/){
         $temp1 = $1;
         $temp2 = $2;
         $temp3 = $3;
         $temp4 = $4;
         $temp5 = $5;
         $temp6 = $6;
         $temp7 = $7;

         if ( $temp1 =~ /^.$/ ) {
            $temp = "0" . $temp1 . ":";
         } 
         else {
            $temp = $temp1 . ":";
         }

         if ( $temp2 =~ /^.$/ ) {
            $temp = $temp . "0" . $temp2 . ":";
         }
         else {
            $temp = $temp . $temp2 . ":";
         }

         if ( $temp3 =~ /^.$/ ) {
            $temp = $temp . "0" . $temp3 . ":";
         }
         else {
            $temp = $temp . $temp3 . ":";
         }

         if ( $temp4 =~ /^.$/ ) {
            $temp = $temp . "0" . $temp4 . ":";
         }
         else {
            $temp = $temp . $temp4 . ":";
         }

         if ( $temp5 =~ /^.$/ ) {
            $temp = $temp . "0" . $temp5 . ":";
         }
         else {
            $temp = $temp . $temp5 . ":";
         }

         if ( $temp6 =~ /^.$/ ) {
            $temp = $temp . "0" . $temp6;
         }
         else {
            $temp = $temp . $temp6;
         }

         $temp =~ tr/a-z/A-Z/;
         $temp = $temp . $temp7;
         print "$temp\n";
      }
   }
}
' $MAP > $TEMPFILE1

# Get MACs that are detected by arpwatch but not in dhcpd.conf.
#./mac-detect.pl $DHCPDCONF $TEMP $OUT_FILE 

perl -e '

#print($ENV{DHCPDCONF});
open(ARPWALK, "$ENV{TEMPFILE1}");
open(TEMPFILE, ">$ENV{TEMPFILE2}");

while ($arp_line = <ARPWALK>) {
    chop $arp_line;
    $arp_line =~ /(^.{17,17})(.*)$/;
    $temp_arp_line = $1;
    $temp_ip_field = $2;
   
    if ($temp_ip_field =~ /^\s*10.*/ ) {
        next;
    }

    open(DHCPCONF, "$ENV{DHCPDCONF}");

    while ($dhcp_line = <DHCPCONF>) {


#	if( ( ($def_gw) = $dhcp_line =~ /option routers\s*(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) and $dhcp_line !~ /}/ and$dhcp_line !~ /10\.10\.\d{1,3}\.\d{1,3}/ ){
#
#		print "$dhcp_line: DEFAULT GATEWAY=$def_gw\n";
# }


#        if ($dhcp_line =~ /^host.*\{ hardware ethernet (.+\:.+\:.+\:.+\:.+\:[A-F0-9]{2}) \;/) {

        if ($dhcp_line =~ /^host.*\{ hardware ethernet ([0-9A-F][0-9A-F]:[0-9A-F][0-9A-F]:[0-9A-F][0-9A-F]:[0-9A-F][0-9A-F]:[0-9A-F][0-9A-F]:[0-9A-F][0-9A-F]) \; .*/) {

		print "=== $1 = $temp_arp_line ==\n";

            if ( $temp_arp_line eq $1 ) {
#                print "ARP MAC2: $temp_arp_line\n";
                $arp_line = " ";
            }
        }
    }

    if ( $arp_line ne " " ) {
       print TEMPFILE "$arp_line\n";
#       print "DHCP1 MAC: $arp_line\n";
    }

    close(DHCPCONF);
}

close(ARPWALK);
print TEMPFILE "\n";
close(TEMPFILE);

open(TEMPFILE, "$ENV{TEMPFILE2}");
open(MAC_REPORT, ">$ENV{OUT_FILE}");

while ($temp_line = <TEMPFILE>) {
   $temp_line =~ /(^.{17,17}).*$/;
   $arp_line = $1;
   open(EXCLUDE, "$ENV{EXCLUDE}");
   while ($exclude_line = <EXCLUDE>) {
     chop $exclude_line;

     if( $exclude_line =~ /^#/){ next;}

     if ( $arp_line eq $exclude_line) {
#        print "$arp_line $exclude_line\n";
        $temp_line = " ";
     }
   }

   if ( $temp_line ne " " ) {
      print MAC_REPORT "$temp_line";
   }
   close(EXCLUDE);
}

close(MAC_REPORT);
close(TEMPFILE);

'

rm -f $TEMPFILE1
rm -f $TEMPFILE2
